@@ -4,20 +4,24 @@ associated with deserialization.
 """
 
 import abc
+import collections
 import dataclasses
 import functools
 import typing
 from dataclasses import dataclass, is_dataclass
-from typing import Any, Callable, Dict, List, Optional, Type, TypeVar
+from typing import Any, Callable, Dict, List, Optional, TypeVar
 
 import jinja2
+from typing_extensions import Type, dataclass_transform
 
 from .compat import (
     Literal,
     SerdeError,
     SerdeSkip,
+    T,
     UserError,
     find_generic_arg,
+    get_args,
     get_generic_arg,
     get_origin,
     has_default,
@@ -28,12 +32,14 @@ from .compat import (
     is_bare_set,
     is_bare_tuple,
     is_datetime,
+    is_default_dict,
     is_dict,
+    is_ellipsis,
     is_enum,
+    is_frozen_set,
     is_generic,
     is_list,
     is_literal,
-    is_new_type_primitive,
     is_none,
     is_opt,
     is_primitive,
@@ -41,6 +47,7 @@ from .compat import (
     is_str_serializable,
     is_tuple,
     is_union,
+    is_variable_tuple,
     iter_literals,
     iter_types,
     iter_unions,
@@ -78,13 +85,13 @@ from .numpy import (
     is_numpy_scalar,
 )
 
-__all__: List = ['deserialize', 'is_deserializable', 'from_dict', 'from_tuple']
+__all__ = ['deserialize', 'is_deserializable', 'from_dict', 'from_tuple']
 
 # Interface of Custom deserialize function.
-DeserializeFunc = Callable[[Type, Any], Any]
+DeserializeFunc = Callable[[Type[Any], Any], Any]
 
 
-def serde_custom_class_deserializer(cls: Type, datavar, value, custom: DeserializeFunc, default: Callable):
+def serde_custom_class_deserializer(cls: Type[Any], datavar, value, custom: DeserializeFunc, default: Callable):
     """
     Handle custom deserialization. Use default deserialization logic if it receives `SerdeSkip` exception.
 
@@ -100,13 +107,22 @@ def serde_custom_class_deserializer(cls: Type, datavar, value, custom: Deseriali
         return default()
 
 
-def default_deserializer(_cls: Type, obj):
+def default_deserializer(_cls: Type[Any], obj):
     """
     Marker function to tell serde to use the default deserializer. It's used when custom deserializer is specified
     at the class but you want to override a field with the default deserializer.
     """
 
 
+def _get_by_aliases(d: Dict[str, str], aliases: List[str]):
+    if not aliases:
+        raise KeyError("Tried all aliases, but key not found")
+    if aliases[0] in d:
+        return d[aliases[0]]
+    else:
+        return _get_by_aliases(d, aliases[1:])
+
+
 def _make_deserialize(
     cls_name: str,
     fields,
@@ -123,7 +139,6 @@ def _make_deserialize(
     C = dataclasses.make_dataclass(cls_name, fields, *args, **kwargs)
     C = deserialize(
         C,
-        *args,
         rename_all=rename_all,
         reuse_instances_default=reuse_instances_default,
         convert_sets_default=convert_sets_default,
@@ -132,10 +147,18 @@ def _make_deserialize(
     return C
 
 
+# The `deserialize` function can call itself recursively when it needs to generate code for
+# unmarked dataclasses. To avoid infinite recursion, this array remembers types for which code is
+# currently being generated.
+GENERATION_STACK = []
+
+
+@dataclass_transform()
 def deserialize(
     _cls=None,
     rename_all: Optional[str] = None,
     reuse_instances_default: bool = True,
+    convert_sets_default: bool = False,
     deserializer: Optional[DeserializeFunc] = None,
     tagging: Tagging = DefaultTagging,
     type_check: TypeCheck = NoCheck,
@@ -198,7 +221,13 @@ def deserialize(
     Foo(i=10, dt=datetime.datetime(2021, 1, 1, 0, 0))
     """
 
+    stack = []
+
     def wrap(cls: Type):
+        if cls in stack:
+            return
+        stack.append(cls)
+
         tagging.check()
 
         # If no `dataclass` found in the class, dataclassify it automatically.
@@ -224,6 +253,7 @@ def deserialize(
         g['typename'] = typename  # used in union functions
         g['ensure'] = ensure
         g['typing'] = typing
+        g['collections'] = collections
         g['Literal'] = Literal
         g['from_obj'] = from_obj
         g['get_generic_arg'] = get_generic_arg
@@ -231,6 +261,7 @@ def deserialize(
         g['TypeCheck'] = TypeCheck
         g['NoCheck'] = NoCheck
         g['coerce'] = coerce
+        g['_get_by_aliases'] = _get_by_aliases
         if deserialize:
             g['serde_custom_class_deserializer'] = functools.partial(
                 serde_custom_class_deserializer, custom=deserializer
@@ -238,7 +269,13 @@ def deserialize(
 
         # Collect types used in the generated code.
         for typ in iter_types(cls):
-            if typ is cls or (is_primitive(typ) and not is_enum(typ) and not is_new_type_primitive(typ)):
+            # When we encounter a dataclass not marked with deserialize, then also generate
+            # deserialize functions for it.
+            if is_dataclass_without_de(typ):
+                # We call deserialize and not wrap to make sure that we will use the default serde
+                # configuration for generating the deserialization function.
+                deserialize(typ)
+            if typ is cls or (is_primitive(typ) and not is_enum(typ)):
                 continue
             if is_generic(typ):
                 g[typename(typ)] = get_origin(typ)
@@ -273,12 +310,20 @@ def deserialize(
 
         logger.debug(f'{typename(cls)}: {SERDE_SCOPE} {scope}')
 
+        stack.pop()
         return cls
 
     if _cls is None:
         return wrap  # type: ignore
 
-    return wrap(_cls)
+    if _cls in GENERATION_STACK:
+        return _cls
+
+    GENERATION_STACK.append(_cls)
+    try:
+        return wrap(_cls)
+    finally:
+        GENERATION_STACK.pop()
 
 
 def is_deserializable(instance_or_class: Any) -> bool:
@@ -295,6 +340,15 @@ def is_deserializable(instance_or_class: Any) -> bool:
     return hasattr(instance_or_class, SERDE_SCOPE)
 
 
+def is_dataclass_without_de(cls: Type[Any]) -> bool:
+    if not dataclasses.is_dataclass(cls):
+        return False
+    if not hasattr(cls, SERDE_SCOPE):
+        return True
+    scope: Optional[SerdeScope] = getattr(cls, SERDE_SCOPE)
+    return FROM_DICT not in scope.funcs
+
+
 class Deserializer(metaclass=abc.ABCMeta):
     """
     `Deserializer` base class. Subclass this to customize deserialize behaviour.
@@ -314,11 +368,18 @@ class Deserializer(metaclass=abc.ABCMeta):
         raise NotImplementedError
 
 
-def from_obj(c: Type, o: Any, named: bool, reuse_instances: bool):
+def from_obj(c: Type[T], o: Any, named: bool, reuse_instances: bool) -> T:
     """
     Deserialize from an object into an instance of the type specified as arg `c`.
     `c` can be either primitive type, `List`, `Tuple`, `Dict` or `deserialize` class.
     """
+
+    def deserializable_to_obj(cls):
+        serde_scope: SerdeScope = getattr(cls, SERDE_SCOPE)
+        func_name = FROM_DICT if named else FROM_ITER
+        res = serde_scope.funcs[func_name](cls, maybe_generic=maybe_generic, data=o, reuse_instances=reuse_instances)
+        return res
+
     if is_generic(c):
         # Store subscripted generic type such as Foo[Bar] in "maybe_generic",
         # and store origin type such as Foo in "c". Since subscripted generics
@@ -332,11 +393,11 @@ def from_obj(c: Type, o: Any, named: bool, reuse_instances: bool):
         thisfunc = functools.partial(from_obj, named=named, reuse_instances=reuse_instances)
         if o is None:
             return None
-        if is_deserializable(c):
-            serde_scope: SerdeScope = getattr(c, SERDE_SCOPE)
-            func_name = FROM_DICT if named else FROM_ITER
-            res = serde_scope.funcs[func_name](c, maybe_generic=maybe_generic, data=o, reuse_instances=reuse_instances)
-            return res
+        if is_dataclass_without_de(c):
+            deserialize(c)
+            return deserializable_to_obj(c)
+        elif is_deserializable(c):
+            return deserializable_to_obj(c)
         elif is_opt(c):
             if o is None:
                 return None
@@ -360,11 +421,14 @@ def from_obj(c: Type, o: Any, named: bool, reuse_instances: bool):
         elif is_set(c):
             if is_bare_set(c):
                 return set(e for e in o)
+            elif is_frozen_set(c):
+                res = frozenset(thisfunc(type_args(c)[0], e) for e in o)
+                return res
             else:
                 res = set(thisfunc(type_args(c)[0], e) for e in o)
                 return res
         elif is_tuple(c):
-            if is_bare_tuple(c):
+            if is_bare_tuple(c) or is_variable_tuple(c):
                 return tuple(e for e in o)
             else:
                 res = tuple(thisfunc(type_args(c)[i], e) for i, e in enumerate(o))
@@ -372,14 +436,22 @@ def from_obj(c: Type, o: Any, named: bool, reuse_instances: bool):
         elif is_dict(c):
             if is_bare_dict(c):
                 return {k: v for k, v in o.items()}
+            elif is_default_dict(c):
+                f = DeField(c, "")
+                v = f.value_field()
+                origin = get_origin(v.type)
+                res = collections.defaultdict(
+                    origin if origin else v.type,
+                    {thisfunc(type_args(c)[0], k): thisfunc(type_args(c)[1], v) for k, v in o.items()},
+                )
             else:
                 res = {thisfunc(type_args(c)[0], k): thisfunc(type_args(c)[1], v) for k, v in o.items()}
-                return res
+            return res
         elif is_numpy_array(c):
             return deserialize_numpy_array_direct(c, o)
         elif is_datetime(c):
             return c.fromisoformat(o)
-        elif is_any(c):
+        elif is_any(c) or is_ellipsis(c):
             return o
 
         return c(o)
@@ -391,7 +463,7 @@ def from_obj(c: Type, o: Any, named: bool, reuse_instances: bool):
         raise SerdeError(e)
 
 
-def from_dict(cls, o, reuse_instances: bool = ...):
+def from_dict(cls: Type[T], o, reuse_instances: bool = ...) -> T:
     """
     Deserialize dictionary into object.
 
@@ -414,7 +486,7 @@ def from_dict(cls, o, reuse_instances: bool = ...):
     return from_obj(cls, o, named=True, reuse_instances=reuse_instances)
 
 
-def from_tuple(cls, o, reuse_instances: bool = ...):
+def from_tuple(cls: Type[T], o: Any, reuse_instances: bool = ...) -> T:
     """
     Deserialize tuple into object.
 
@@ -458,6 +530,9 @@ class DeField(Field):
             'skip': self.skip,
             'skip_if': self.skip_if,
             'skip_if_false': self.skip_if_false,
+            'flatten': self.flatten,
+            'alias': self.alias,
+            'parent': self.parent,
         }
         if is_list(self.type) or is_dict(self.type) or is_set(self.type):
             return InnerField(typ, 'v', datavar='v', **opts)
@@ -485,6 +560,8 @@ class DeField(Field):
     def data(self) -> str:
         if self.iterbased:
             return f'{self.datavar}[{self.index}]'
+        elif is_union(self.type) and type(None) in get_args(self.type):
+            return f'{self.datavar}.get("{self.conv_name()}")'
         else:
             return f'{self.datavar}["{self.conv_name()}"]'
 
@@ -514,7 +591,8 @@ class InnerField(DeField):
         self.datavar = d
 
 
-defields = functools.partial(fields, DeField)
+def defields(cls: Type[Any]) -> List[DeField]:
+    return fields(DeField, cls)
 
 
 @dataclass
@@ -567,7 +645,7 @@ class Renderer:
                     if reuse_instances else {from_iso}"
         elif is_none(arg.type):
             res = "None"
-        elif arg.type is Any:
+        elif is_any(arg.type) or is_ellipsis(arg.type):
             res = arg.data
         elif isinstance(arg.type, TypeVar):
             index = find_generic_arg(self.cls, arg.type)
@@ -623,8 +701,14 @@ class Renderer:
             else:
                 var = arg.datavar
 
-        opts = "reuse_instances=reuse_instances"
-        return f"{typename(arg.type)}.{SERDE_SCOPE}.funcs['{self.func}'](data={var}, {opts})"
+        opts = "maybe_generic=maybe_generic, reuse_instances=reuse_instances"
+
+        if arg.is_self_referencing():
+            class_name = "cls"
+        else:
+            class_name = typename(arg.type)
+
+        return f"{class_name}.{SERDE_SCOPE}.funcs['{self.func}'](data={var}, {opts})"
 
     def opt(self, arg: DeField) -> str:
         """
@@ -644,18 +728,15 @@ class Renderer:
         ... class Foo:
         ...     o: Optional[List[int]]
         >>> Renderer('foo').render(DeField(Optional[Foo], 'f', datavar='data'))
-        '(Foo.__serde__.funcs[\\'foo\\'](data=data["f"], reuse_instances=reuse_instances)) \
-if data.get("f") is not None else None'
+        '(Foo.__serde__.funcs[\\'foo\\'](data=data["f"], maybe_generic=maybe_generic, \
+reuse_instances=reuse_instances)) if data.get("f") is not None else None'
         """
         value = arg[0]
-        if has_default(arg):
-            return self.render(value)
+        if arg.iterbased:
+            exists = f'{arg.data} is not None'
         else:
-            if arg.iterbased:
-                exists = f'{arg.data} is not None'
-            else:
-                exists = f'{arg.datavar}.get("{arg.conv_name()}") is not None'
-            return f'({self.render(value)}) if {exists} else None'
+            exists = f'{arg.datavar}.get("{arg.conv_name()}") is not None'
+        return f'({self.render(value)}) if {exists} else None'
 
     def list(self, arg: DeField) -> str:
         """
@@ -686,6 +767,8 @@ if data.get("f") is not None else None'
         """
         if is_bare_set(arg.type):
             return f'set({arg.data})'
+        elif is_frozen_set(arg.type):
+            return f'frozenset({self.render(arg[0])} for v in {arg.data})'
         else:
             return f'set({self.render(arg[0])} for v in {arg.data})'
 
@@ -699,15 +782,15 @@ if data.get("f") is not None else None'
         >>> Renderer('foo').render(DeField(Tuple[str, int, List[int], Foo], 'd', datavar='data'))
         '(coerce(str, data["d"][0]), coerce(int, data["d"][1]), \
 [coerce(int, v) for v in data["d"][2]], \
-Foo.__serde__.funcs[\\'foo\\'](data=data["d"][3], reuse_instances=reuse_instances),)'
+Foo.__serde__.funcs[\\'foo\\'](data=data["d"][3], maybe_generic=maybe_generic, reuse_instances=reuse_instances),)'
 
         >>> field = DeField(Tuple[str, int, List[int], Foo], 'd', datavar='data', index=0, iterbased=True)
         >>> Renderer('foo').render(field)
         "(coerce(str, data[0][0]), coerce(int, data[0][1]), \
 [coerce(int, v) for v in data[0][2]], Foo.__serde__.funcs['foo'](data=data[0][3], \
-reuse_instances=reuse_instances),)"
+maybe_generic=maybe_generic, reuse_instances=reuse_instances),)"
         """
-        if is_bare_tuple(arg.type):
+        if is_bare_tuple(arg.type) or is_variable_tuple(arg.type):
             return f'tuple({arg.data})'
         else:
             values = []
@@ -727,11 +810,25 @@ reuse_instances=reuse_instances),)"
         >>> @deserialize
         ... class Foo: pass
         >>> Renderer('foo').render(DeField(Dict[Foo, List[Foo]], 'f', datavar='data'))
-        '{Foo.__serde__.funcs[\\'foo\\'](data=k, reuse_instances=reuse_instances): \
-[Foo.__serde__.funcs[\\'foo\\'](data=v, reuse_instances=reuse_instances) for v in v] for k, v in data["f"].items()}'
+        '{Foo.__serde__.funcs[\\'foo\\'](data=k, maybe_generic=maybe_generic, reuse_instances=reuse_instances): \
+[Foo.__serde__.funcs[\\'foo\\'](data=v, maybe_generic=maybe_generic, reuse_instances=reuse_instances) for v in v] \
+for k, v in data["f"].items()}'
         """
         if is_bare_dict(arg.type):
             return arg.data
+        elif is_default_dict(arg.type):
+            k = arg.key_field()
+            v = arg.value_field()
+            origin = get_origin(v.type)
+            if origin:
+                # When the callable type is of generic type e.g List.
+                # Get origin type "list" from "List[X]".
+                callable = origin.__name__
+            else:
+                # When the callable type is non generic type e.g int, Foo.
+                callable = v.type.__name__
+            return f'collections.defaultdict({callable}, \
+                    {{{self.render(k)}: {self.render(v)} for k, v in {arg.data}.items()}})'
         else:
             k = arg.key_field()
             v = arg.value_field()
@@ -755,6 +852,9 @@ reuse_instances=reuse_instances),)"
         """
         typ = typename(arg.type)
         dat = arg.data
+        if arg.alias:
+            aliases = map(lambda s: f'"{s}"', [arg.name, *arg.alias])
+            dat = f"_get_by_aliases(data, [{','.join(aliases)}])"
         if self.suppress_coerce:
             return dat
         else:
@@ -786,7 +886,7 @@ reuse_instances=reuse_instances),)"
             return code
 
 
-def to_arg(f: DeField, index, rename_all: Optional[str] = None) -> DeField:
+def to_arg(f: DeField, index: int, rename_all: Optional[str] = None) -> DeField:
     f.index = index
     f.data = 'data'
     f.case = f.case or rename_all
@@ -799,7 +899,7 @@ def to_iter_arg(f: DeField, *args, **kwargs) -> DeField:
     return f
 
 
-def render_from_iter(cls: Type, custom: Optional[DeserializeFunc] = None, type_check: TypeCheck = NoCheck) -> str:
+def render_from_iter(cls: Type[Any], custom: Optional[DeserializeFunc] = None, type_check: TypeCheck = NoCheck) -> str:
     template = """
 def {{func}}(cls=cls, maybe_generic=None, data=None, reuse_instances = {{serde_scope.reuse_instances_default}}):
   if reuse_instances is Ellipsis:
@@ -835,7 +935,7 @@ def {{func}}(cls=cls, maybe_generic=None, data=None, reuse_instances = {{serde_s
 
 
 def render_from_dict(
-    cls: Type,
+    cls: Type[Any],
     rename_all: Optional[str] = None,
     custom: Optional[DeserializeFunc] = None,
     type_check: TypeCheck = NoCheck,
@@ -883,7 +983,7 @@ def {{func}}(cls=cls, maybe_generic=None, data=None,
     return res
 
 
-def render_union_func(cls: Type, union_args: List[Type], tagging: Tagging = DefaultTagging) -> str:
+def render_union_func(cls: Type[Any], union_args: List[Type], tagging: Tagging = DefaultTagging) -> str:
     template = """
 def {{func}}(cls=cls, maybe_generic=None, data=None, reuse_instances = {{serde_scope.reuse_instances_default}}):
   errors = []
@@ -909,7 +1009,7 @@ def {{func}}(cls=cls, maybe_generic=None, data=None, reuse_instances = {{serde_s
     fake_dict = {"fake_key": data}
     {% endif %}
 
-    {% if t|is_primitive or t |is_none %}
+    {% if t|is_primitive or t|is_none %}
     if not isinstance(fake_dict["fake_key"], {{t|typename}}):
         raise Exception("Not a type of {{t|typename}}")
     {% endif %}
@@ -941,7 +1041,7 @@ def {{func}}(cls=cls, maybe_generic=None, data=None, reuse_instances = {{serde_s
     )
 
 
-def render_literal_func(cls: Type, literal_args: List[Any], tagging: Tagging = DefaultTagging) -> str:
+def render_literal_func(cls: Type[Any], literal_args: List[Any], tagging: Tagging = DefaultTagging) -> str:
     template = """
 def {{func}}(cls=cls, maybe_generic=None, data=None, reuse_instances = {{serde_scope.reuse_instances_default}}):
   if data in ({%- for v in literal_args -%}{{v|repr}},{%- endfor -%}):
