@@ -9,20 +9,23 @@ import dataclasses
 import functools
 import typing
 from dataclasses import dataclass, is_dataclass
-from typing import Any, Callable, Dict, Iterator, List, Optional, Type, TypeVar
+from typing import Any, Callable, Dict, Iterator, List, Optional, Tuple, Type, TypeVar
 
 import jinja2
+from typing_extensions import dataclass_transform
 
 from .compat import (
     Literal,
     SerdeError,
     SerdeSkip,
     get_origin,
+    is_any,
     is_bare_dict,
     is_bare_list,
     is_bare_opt,
     is_bare_set,
     is_bare_tuple,
+    is_class_var,
     is_datetime,
     is_datetime_instance,
     is_dict,
@@ -30,7 +33,6 @@ from .compat import (
     is_generic,
     is_list,
     is_literal,
-    is_new_type_primitive,
     is_none,
     is_opt,
     is_primitive,
@@ -39,6 +41,7 @@ from .compat import (
     is_str_serializable_instance,
     is_tuple,
     is_union,
+    is_variable_tuple,
     iter_types,
     iter_unions,
     type_args,
@@ -78,17 +81,17 @@ from .numpy import (
 __all__ = ["serialize", "is_serializable", "to_dict", "to_tuple"]
 
 # Interface of Custom serialize function.
-SerializeFunc = Callable[[Type, Any], Any]
+SerializeFunc = Callable[[Type[Any], Any], Any]
 
 
-def default_serializer(_cls: Type, obj):
+def default_serializer(_cls: Type[Any], obj):
     """
     Marker function to tell serde to use the default serializer. It's used when custom serializer is specified
     at the class but you want to override a field with the default serializer.
     """
 
 
-def serde_custom_class_serializer(cls: Type, obj, custom: SerializeFunc, default: Callable):
+def serde_custom_class_serializer(cls: Type[Any], obj: Any, custom: SerializeFunc, default: Callable):
     try:
         return custom(cls, obj)
     except SerdeSkip:
@@ -115,6 +118,9 @@ def _make_serialize(
     reuse_instances_default: bool = True,
     convert_sets_default: bool = False,
     serializer: Optional[SerializeFunc] = None,
+    tagging: Tagging = DefaultTagging,
+    type_check: TypeCheck = NoCheck,
+    serialize_class_var: bool = False,
     **kwargs,
 ):
     """
@@ -123,15 +129,25 @@ def _make_serialize(
     C = dataclasses.make_dataclass(cls_name, fields, *args, **kwargs)
     C = serialize(
         C,
-        *args,
         rename_all=rename_all,
         reuse_instances_default=reuse_instances_default,
         convert_sets_default=convert_sets_default,
+        serializer=serializer,
+        tagging=tagging,
+        type_check=type_check,
+        serialize_class_var=serialize_class_var,
         **kwargs,
     )
     return C
 
 
+# The `serialize` function can call itself recursively when it needs to generate code for
+# unmarked dataclasses. To avoid infinite recursion, this array remembers types for which code is
+# currently being generated.
+GENERATION_STACK = []
+
+
+@dataclass_transform()
 def serialize(
     _cls=None,
     rename_all: Optional[str] = None,
@@ -140,6 +156,7 @@ def serialize(
     serializer: Optional[SerializeFunc] = None,
     tagging: Tagging = DefaultTagging,
     type_check: TypeCheck = NoCheck,
+    serialize_class_var: bool = False,
     **kwargs,
 ):
     """
@@ -197,9 +214,19 @@ def serialize(
 
     >>> to_json(Foo(10, datetime(2021, 1, 1, 0, 0, 0)))
     '{"i":10,"dt":"01/01/21"}'
+
+    * `serialize_class_var` enables `typing.ClassVar` serialization.
+
+    >>> @serialize(serialize_class_var=True)
+    ... class Foo:
+    ...     v: typing.ClassVar[int] = 10
+    >>>
+    >>> to_json(Foo())
+    '{"v":10}'
+
     """
 
-    def wrap(cls: Type):
+    def wrap(cls: Type[Any]):
         tagging.check()
 
         # If no `dataclass` found in the class, dataclassify it automatically.
@@ -241,7 +268,14 @@ def serialize(
 
         # Collect types used in the generated code.
         for typ in iter_types(cls):
-            if typ is cls or (is_primitive(typ) and not is_enum(typ) and not is_new_type_primitive(typ)):
+            # When we encounter a dataclass not marked with serialize, then also generate serialize
+            # functions for it.
+            if is_dataclass_without_se(typ):
+                # We call serialize and not wrap to make sure that we will use the default serde
+                # configuration for generating the serialization function.
+                serialize(typ)
+
+            if typ is cls or (is_primitive(typ) and not is_enum(typ)):
                 continue
             g[typename(typ)] = typ
 
@@ -252,14 +286,14 @@ def serialize(
             add_func(scope, union_key, render_union_func(cls, union_args, tagging), g)
             scope.union_se_args[union_key] = union_args
 
-        for f in sefields(cls):
+        for f in sefields(cls, serialize_class_var):
             if f.skip_if:
                 g[f.skip_if.name] = f.skip_if
             if f.serializer:
                 g[f.serializer.name] = f.serializer
 
-        add_func(scope, TO_ITER, render_to_tuple(cls, serializer, type_check), g)
-        add_func(scope, TO_DICT, render_to_dict(cls, rename_all, serializer, type_check), g)
+        add_func(scope, TO_ITER, render_to_tuple(cls, serializer, type_check, serialize_class_var), g)
+        add_func(scope, TO_DICT, render_to_dict(cls, rename_all, serializer, type_check, serialize_class_var), g)
         add_func(scope, TYPE_CHECK, render_type_check(cls), g)
 
         logger.debug(f"{typename(cls)}: {SERDE_SCOPE} {scope}")
@@ -269,7 +303,14 @@ def serialize(
     if _cls is None:
         return wrap  # type: ignore
 
-    return wrap(_cls)
+    if _cls in GENERATION_STACK:
+        return _cls
+
+    GENERATION_STACK.append(_cls)
+    try:
+        return wrap(_cls)
+    finally:
+        GENERATION_STACK.pop()
 
 
 def is_serializable(instance_or_class: Any) -> bool:
@@ -291,7 +332,21 @@ def is_serializable(instance_or_class: Any) -> bool:
     return hasattr(instance_or_class, SERDE_SCOPE)
 
 
-def to_obj(o, named: bool, reuse_instances: bool, convert_sets: bool, c: Type = None):
+def is_dataclass_without_se(cls: Type[Any]) -> bool:
+    if not dataclasses.is_dataclass(cls):
+        return False
+    if not hasattr(cls, SERDE_SCOPE):
+        return True
+    scope: Optional[SerdeScope] = getattr(cls, SERDE_SCOPE)
+    return TO_DICT not in scope.funcs
+
+
+def to_obj(o, named: bool, reuse_instances: bool, convert_sets: bool, c: Optional[Type[Any]] = None):
+    def serializable_to_obj(object):
+        serde_scope: SerdeScope = getattr(object, SERDE_SCOPE)
+        func_name = TO_DICT if named else TO_ITER
+        return serde_scope.funcs[func_name](object, reuse_instances=reuse_instances, convert_sets=convert_sets)
+
     try:
         thisfunc = functools.partial(
             to_obj,
@@ -301,15 +356,11 @@ def to_obj(o, named: bool, reuse_instances: bool, convert_sets: bool, c: Type =
         )
         if o is None:
             return None
+        if is_dataclass_without_se(o):
+            serialize(type(o))
+            return serializable_to_obj(o)
         if is_serializable(o):
-            serde_scope: SerdeScope = getattr(o, SERDE_SCOPE)
-            func_name = TO_DICT if named else TO_ITER
-            return serde_scope.funcs[func_name](o, reuse_instances=reuse_instances, convert_sets=convert_sets)
-        elif is_dataclass(o):
-            if named:
-                return dataclasses.asdict(o)
-            else:
-                return dataclasses.astuple(o)
+            return serializable_to_obj(o)
         elif isinstance(o, list):
             return [thisfunc(e) for e in o]
         elif isinstance(o, tuple):
@@ -329,14 +380,14 @@ def to_obj(o, named: bool, reuse_instances: bool, convert_sets: bool, c: Type =
         raise SerdeError(e)
 
 
-def astuple(v):
+def astuple(v: Any) -> Tuple[Any, ...]:
     """
     Serialize object into tuple.
     """
     return to_tuple(v, reuse_instances=False, convert_sets=False)
 
 
-def to_tuple(o, reuse_instances: bool = ..., convert_sets: bool = ...) -> Any:
+def to_tuple(o, reuse_instances: bool = ..., convert_sets: bool = ...) -> Tuple[Any, ...]:
     """
     Serialize object into tuple.
 
@@ -359,14 +410,14 @@ def to_tuple(o, reuse_instances: bool = ..., convert_sets: bool = ...) -> Any:
     return to_obj(o, named=False, reuse_instances=reuse_instances, convert_sets=convert_sets)
 
 
-def asdict(v):
+def asdict(v: Any) -> Dict[Any, Any]:
     """
     Serialize object into dictionary.
     """
     return to_dict(v, reuse_instances=False, convert_sets=False)
 
 
-def to_dict(o, reuse_instances: bool = ..., convert_sets: bool = ...) -> Any:
+def to_dict(o, reuse_instances: bool = ..., convert_sets: bool = ...) -> Dict[Any, Any]:
     """
     Serialize object into dictionary.
 
@@ -410,22 +461,26 @@ class SeField(Field):
                 raise SerdeError("Field name is None.")
             return self.name
 
-    def __getitem__(self, n) -> "SeField":
+    def __getitem__(self, n: int) -> "SeField":
         typ = type_args(self.type)[n]
         return SeField(typ, name=None)
 
 
-def sefields(cls: Type) -> Iterator[SeField]:
+def sefields(cls: Type[Any], serialize_class_var: bool = False) -> Iterator[SeField]:
     """
     Iterate fields for serialization.
     """
-    for f in fields(SeField, cls):
+    for f in fields(SeField, cls, serialize_class_var=serialize_class_var):
         f.parent = SeField(None, "obj")  # type: ignore
-        assert isinstance(f, SeField)
         yield f
 
 
-def render_to_tuple(cls: Type, custom: Optional[SerializeFunc] = None, type_check: TypeCheck = NoCheck) -> str:
+def render_to_tuple(
+    cls: Type[Any],
+    custom: Optional[SerializeFunc] = None,
+    type_check: TypeCheck = NoCheck,
+    serialize_class_var: bool = False,
+) -> str:
     template = """
 def {{func}}(obj, reuse_instances = {{serde_scope.reuse_instances_default}},
              convert_sets = {{serde_scope.convert_sets_default}}):
@@ -450,16 +505,25 @@ def {{func}}(obj, reuse_instances = {{serde_scope.reuse_instances_default}},
   )
     """
 
-    renderer = Renderer(TO_ITER, custom, suppress_coerce=(not type_check.is_coerce()))
+    renderer = Renderer(
+        TO_ITER, custom, suppress_coerce=(not type_check.is_coerce()), serialize_class_var=serialize_class_var
+    )
     env = jinja2.Environment(loader=jinja2.DictLoader({"iter": template}))
     env.filters.update({"rvalue": renderer.render})
     return env.get_template("iter").render(
-        func=TO_ITER, serde_scope=getattr(cls, SERDE_SCOPE), fields=sefields(cls), type_check=type_check
+        func=TO_ITER,
+        serde_scope=getattr(cls, SERDE_SCOPE),
+        fields=sefields(cls, serialize_class_var),
+        type_check=type_check,
     )
 
 
 def render_to_dict(
-    cls: Type, case: Optional[str] = None, custom: Optional[SerializeFunc] = None, type_check: TypeCheck = NoCheck
+    cls: Type[Any],
+    case: Optional[str] = None,
+    custom: Optional[SerializeFunc] = None,
+    type_check: TypeCheck = NoCheck,
+    serialize_class_var: bool = False,
 ) -> str:
     template = """
 def {{func}}(obj, reuse_instances = {{serde_scope.reuse_instances_default}},
@@ -491,17 +555,20 @@ def {{func}}(obj, reuse_instances = {{serde_scope.reuse_instances_default}},
   return res
     """
     renderer = Renderer(TO_DICT, custom, suppress_coerce=(not type_check.is_coerce()))
-    lrenderer = LRenderer(case)
+    lrenderer = LRenderer(case, serialize_class_var)
     env = jinja2.Environment(loader=jinja2.DictLoader({"dict": template}))
     env.filters.update({"rvalue": renderer.render})
     env.filters.update({"lvalue": lrenderer.render})
     env.filters.update({"case": functools.partial(conv, case=case)})
     return env.get_template("dict").render(
-        func=TO_DICT, serde_scope=getattr(cls, SERDE_SCOPE), fields=sefields(cls), type_check=type_check
+        func=TO_DICT,
+        serde_scope=getattr(cls, SERDE_SCOPE),
+        fields=sefields(cls, serialize_class_var),
+        type_check=type_check,
     )
 
 
-def render_union_func(cls: Type, union_args: List[Type], tagging: Tagging = DefaultTagging) -> str:
+def render_union_func(cls: Type[Any], union_args: List[Type[Any]], tagging: Tagging = DefaultTagging) -> str:
     """
     Render function that serializes a field with union type.
     """
@@ -554,6 +621,7 @@ class LRenderer:
     """
 
     case: Optional[str]
+    serialize_class_var: bool = False
 
     def render(self, arg: SeField) -> str:
         """
@@ -569,7 +637,7 @@ class LRenderer:
         Render field with flatten attribute.
         """
         flattened = []
-        for f in sefields(arg.type):
+        for f in sefields(arg.type, self.serialize_class_var):
             flattened.append(self.render(f))
         return ", ".join(flattened)
 
@@ -584,6 +652,7 @@ class Renderer:
     custom: Optional[SerializeFunc] = None  # Custom class level serializer.
     suppress_coerce: bool = False
     """ Suppress type coercing because generated union serializer has its own type checking """
+    serialize_class_var: bool = False
 
     def render(self, arg: SeField) -> str:
         """
@@ -651,13 +720,18 @@ convert_sets=convert_sets), coerce(int, foo[2]),)"
             res = f"{arg.varname} if reuse_instances else {arg.varname}.isoformat()"
         elif is_none(arg.type):
             res = "None"
-        elif arg.type is Any or isinstance(arg.type, TypeVar):
+        elif is_any(arg.type) or isinstance(arg.type, TypeVar):
             res = f"to_obj({arg.varname}, True, False, False)"
         elif is_generic(arg.type):
-            arg.type = get_origin(arg.type)
+            origin = get_origin(arg.type)
+            assert origin
+            arg.type = origin
             res = self.render(arg)
         elif is_literal(arg.type):
             res = self.literal(arg)
+        elif is_class_var(arg.type):
+            arg.type = type_args(arg.type)[0]
+            res = self.render(arg)
         else:
             res = f"raise_unsupported_type({arg.varname})"
 
@@ -680,7 +754,7 @@ convert_sets=convert_sets), coerce(int, foo[2]),)"
         """
         if arg.flatten:
             flattened = []
-            for f in sefields(arg.type):
+            for f in sefields(arg.type, self.serialize_class_var):
                 f.parent = arg  # type: ignore
                 flattened.append(self.render(f))  # type: ignore
             return ", ".join(flattened)
@@ -730,7 +804,7 @@ convert_sets=convert_sets), coerce(int, foo[2]),)"
         """
         Render rvalue for tuple.
         """
-        if is_bare_tuple(arg.type):
+        if is_bare_tuple(arg.type) or is_variable_tuple(arg.type):
             return arg.varname
         else:
             rvalues = []
@@ -778,7 +852,7 @@ convert_sets=convert_sets), coerce(int, foo[2]),)"
         return f"{arg.varname}"
 
 
-def enum_value(cls, e):
+def enum_value(cls: Any, e: Any) -> Any:
     """
     Helper function to get value from enum or enum compatible value.
     """
